# Challenge : we have a tiny data of 11 socks => we cannot use a machine learning algorithm to solve this

# Experiment with 11 orphan socks

  n_picked <- 11 # number of socks we picked

  # Apply ABC methodology
  sock_sim <- replicate(1e5, {
  # Generating a sample of the parameters from the priors
    # Prior on n_socks
    prior_mu <- 30
    prior_sd <- 15
    prior_size <- -prior_mu^2 / (prior_mu - prior_sd^2)
    n_socks <- rnbinom(1, mu = prior_mu, size = prior_size)
    
    # Conditional prior on n_pairs knowing n_socks
    prop_pairs <- rbeta(1, shape1 = 15, shape2 = 2)
    n_pairs <- round(floor(n_socks / 2) * prop_pairs)
    n_odd <- n_socks - n_pairs * 2
    
    # Simulating picking out n_picked socks
    socks <- rep(seq_len(n_pairs + n_odd), rep(c(2, 1), c(n_pairs, n_odd)))
    picked_socks <- sample(socks, size =  min(n_picked, n_socks))
    sock_counts <- table(picked_socks)
    
    # Returning the parameters and counts of the number of matched 
    # and unique socks among those that were picked out.
    c(unique = sum(sock_counts == 1), pairs = sum(sock_counts == 2),
      n_socks = n_socks, n_pairs = n_pairs, n_odd = n_odd, prop_pairs = prop_pairs)
  })

    # just translating sock_sim to get one variable per column
    sock_sim <- t(sock_sim)
    head(sock_sim)

   # Generate the estimates posterior distribution
    post_samples <- sock_sim[sock_sim[, "unique"] == 11 &
                               sock_sim[, "pairs" ] == 0 , ]
    
  # Generate the true posterior distribution
#    post_samples_real = c()
#    sock_sim_T <- sock_sim[sock_sim[,3] > 10,]
#    for (i in 1:length(sock_sim_T))
#    {
#     post_samples_real[i] <- rhyper(nn = 1, 
#                                  m = sock_sim_T[i,5],
#                                 n = sock_sim_T[i,4], 
#                                  k = n_picked) 
#      
#    }

    
   # Plot prior and posterior distributions 
    # Priors
  par(mfrow=c(2,2))
  hist(sock_sim[,3], col = "green", xlab = "Number of socks", ylab = "",breaks = 30, freq = FALSE, 
       main = "Prior on n_socks")
  hist(sock_sim[,4], col = "green", xlab = "Number of sock pairs", ylab = "",breaks = 30, freq = FALSE, 
       main = "Resulting prior on n_pairs")
  hist(sock_sim[,6], col = "green", xlab = "Proportion of socks in pairs", ylab = "",breaks = 30, freq = FALSE, 
       main = "Prior on prop_pairs")
  hist(sock_sim[,5], col = "green", xlab = "Number of odd stocks", ylab = "",breaks = 30, freq = FALSE, 
       main = "Resulting prior on n_odd")
  
    # Posteriors
  par(mfrow=c(2,2))
  hist(post_samples[,3], col = "blue", xlab = "Number of socks", ylab = "",breaks = 30, freq = FALSE, 
       main = "Posterior on n_socks")
  hist(post_samples[,4], col = "blue", xlab = "Number of sock pairs", ylab = "",breaks = 30, freq = FALSE, 
       main = "Resulting posterior on n_pairs")
  hist(post_samples[,6], col = "blue", xlab = "Proportion of socks in pairs", ylab = "",breaks = 30, freq = FALSE, 
       main = "Posterior on prop_pairs")
  hist(post_samples[,5], col = "blue", xlab = "Number of odd stocks", ylab = "",breaks = 30, freq = FALSE, 
       main = "Resulting posterior on n_odd")
  
  
  # Determine the parameters that best fit the data
  n_socks_real <- round(mean(post_samples[,3]),0)
  n_pairs_real <- round(mean(post_samples[,4]),0)
  n_odd_real <- round(mean(post_samples[,5]),0)
  
  cat(paste("Total number of socks:", n_socks_real),
      paste("Number of socks in pairs:" , n_pairs_real),
      paste("Number of orphans:" , n_odd_real),
      sep = "\n")
  

  
  
    
  
    




